@@ -75,9 +75,6 @@ class ForecasterBase(object):
         self.categorical_features = categorical_features.copy()
         self.calendar_anomaly = calendar_anomaly
         self.ts_uid_columns = ts_uid_columns
-        for ts_uid_column in ts_uid_columns:  
-            if ts_uid_column in categorical_features: continue
-            self.categorical_features[ts_uid_column] = "default"
         self.trend_models = trend_models
         self.target_scaler = target_scaler
         self.lags = lags
@@ -280,7 +277,7 @@ class ForecasterBase(object):
                     predict_features["_"+feature] = transformed.values
                 else:
                     del predict_features[feature]
-                    predict_features[feature] = transformed.values
+                    predict_features[transformed.columns] = transformed.values
         features_to_keep = [feature for feature in predict_features.columns if feature in self.raw_features]
         return predict_features.loc[:, features_to_keep]
     
